########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/o_agesum.4gs/midlevel.4gl  1.6  Delta: 7/30/90
########################################################################

globals "globals.4gl"

#######################################################################
function ml_join()
#######################################################################
#
    let ct.sel_join =
        "strcustr.cust_code = stropend.cust_code and ",
        "strcustr.acct_bal != 0 and ",
        "stropend.balance != 0"
    return
end function
# ml_join()

#######################################################################
function ml_filter()
#######################################################################
#
    define
        quote char(1)

    call ar_aged() returning rpt.aging_date
    call sel_cust() returning ct.sel_filter

    let quote = "\""

    if m_rcntrc.age_datetype = "I"
    then
        let ct.sel_filter = ct.sel_filter clipped,
            " and inv_date <= ", quote, rpt.aging_date, quote
    else
        let ct.sel_filter = ct.sel_filter clipped,
            " and due_date <= ", quote, rpt.aging_date, quote
    end if
    return
end function
# ml_filter()

#######################################################################
function ml_order()
#######################################################################
#
    let ct.sel_order = report_sort clipped, ", stropend.doc_no "
    return
end function
# ml_order()

#######################################################################
function ml_getcount()
#######################################################################
#
    let ct.sel_stmt =
        "select count(*) ",
        "from strcustr, outer stropend ",
        "where (", ct.sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    whenever error continue
    prepare get_count from ct.sel_stmt
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into ct.num_rows
    close count_cursor
    whenever error call error_handler
    return
end function
# ml_getcount()

#######################################################################
function ml_define_cur()
#######################################################################
#
    let ct.sel_stmt =
        "select ",
            "stropend.inv_date, ",
            "stropend.balance, ",
            "stropend.due_date, ",
            "strcustr.on_acct_amt, ",
            "strcustr.cust_code, ",
            "strcustr.bus_name, ",
            "stropend.inv_no, ",
            "stropend.doc_no ",
        "from strcustr, outer stropend ",
        "where (", ct.sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    if ct.sel_order is not null
    then
        let ct.sel_stmt = ct.sel_stmt clipped,
        " order by ", ct.sel_order clipped
    end if
    whenever error continue
    prepare get_curs from ct.sel_stmt
    declare rpt_cursor cursor for get_curs
    open rpt_cursor
    whenever error call error_handler
    return
end function
# ml_define_cur()

#######################################################################
function ml_fetch()
#######################################################################
#
    fetch rpt_cursor into curs_next.*
    return
end function
# ml_fetch()

#######################################################################
function ml_curs_null()
#######################################################################
#  this function sets all of the elements in the curs
#  record to null
#
    initialize curs.* to null
end function
# ml_curs_null()

#######################################################################
function ml_next_null()
#######################################################################
#  this function sets all of the elements in the curs_next
#  record to null
#
    initialize curs_next.* to null
end function
# ml_next_null()

#######################################################################
function ml_curs_prep()
#######################################################################
#  this function transfers data from the curs_next record to the
#  curs record.
#
    let curs.inv_date = curs_next.inv_date
    let curs.balance = curs_next.balance
    let curs.due_date = curs_next.due_date
    let curs.on_acct_amt = curs_next.on_acct_amt
    let curs.cust_code = curs_next.cust_code
    let curs.bus_name = curs_next.bus_name
    let curs.inv_no = curs_next.inv_no
end function
# ml_curs_prep()


#######################################################################
function ml_before_break()
#######################################################################
#  this function calls before_group() when appropriate.
#
    define
        sort_cur char(37),
        sort_cur_next char(37),
        no_break    smallint    # set to false if control break
    let no_break = true
    case
        when report_sort = "cust_code"
             let sort_cur = curs.cust_code
             let sort_cur_next = curs_next.cust_code
        otherwise
             let sort_cur = curs.bus_name,",",curs.cust_code
             let sort_cur_next = curs_next.bus_name,",",curs_next.cust_code
    end case
    if no_break and sort_cur = sort_cur_next
    then else
        if no_break then call ml_curs_prep() end if
        call before_group(report_sort)
        let no_break = false
    end if
    if no_break and curs.inv_no = curs_next.inv_no
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("inv_no")
        let no_break = false
    end if
end function
# ml_before_break()

#######################################################################
function ml_after_break()
#######################################################################
#  this function calls after_group() when appropriate.
#
    define
        sort_cur char(37),
        sort_cur_next char(37),
        no_break    smallint    # set to false if control break
    let no_break = true
    case
        when report_sort = "cust_code"
             let sort_cur = curs.cust_code
             let sort_cur_next = curs_next.cust_code
        otherwise
             let sort_cur = curs.bus_name,",",curs.cust_code
             let sort_cur_next = curs_next.bus_name,",",curs_next.cust_code
    end case
    if no_break and sort_cur = sort_cur_next 
    then else
        call after_group(report_sort)
        let no_break = false
    end if
    if no_break and curs.inv_no = curs_next.inv_no
    then else
        call after_group("inv_no")
        let no_break = false
    end if
end function
# ml_after_break()

#######################################################################
function ml_output()
#######################################################################
#
    output to report report1 (rpt.*)
    return
end function
# ml_output()



#######################################################################
function ml_defaults()
#######################################################################
#  this function sets the ct.* record values
#
    define
        n       smallint    # working number

    # default to edit list
    let report_sort = "cust_code"
    let ct.prcname = "Customer Aging Summary By Code"
    for n = 1 to num_args()
        if arg_val(n) = "-d"
        then
            let report_sort = "cust_code"
            let ct.prcname = "Customer Aging Summary By Code"
        end if
        if arg_val(n) = "-n"
        then
            let report_sort = "bus_name"
            let ct.prcname = "Customer Aging Summary By Name"
        end if
    end for
    let rpt.heading = ct.prcname
    let ct.rtmargn = "FourGen Modifiable Accounting"
    let ct.destin = "report.out"
    let ct.quiet = "10"
    let rpt.co_name = al_compn()
end function
# ml_defaults()
