##############################################################################
# Copyright 1988 Fourgen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/o_statmt.4gs/lowlevel.4gl  1.4  Last Delta: 8/31/90
##############################################################################

globals "globals.4gl"

#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "first_row" then
        # insert "on first row" code here
        return
    end if

    if group_key = "cust_code" then
        # Flow:
        # - set rpt line to nulls
        # - if balfwd customer, process all items since last statement
        # - if openitm customer, process all open invoices in stropend
        # - "process" means to fill the rpt vars & send to report
        #   (note: all numeric fields using "----,---.&&")
        call on_every_row()    # setup the header variables
        call clear_line()      # blank out the rpt line
        let cust_total = 0     # customer total accumulator
        let rpt.coment = "CONTINUED ON NEXT PAGE"
        let rpt.plspay = "***********"
        if curs.ar_type = "B"
        then
            call balance_forward()
        else
            call open_item()
        end if
        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    let rpt.cust_code = curs.cust_code
    let rpt.bus_name = curs.bus_name
    let rpt.address1 = curs.address1
    let rpt.address2 = curs.address2
    let rpt.stmt_date = entry_date using "mm/dd/yy"
    let rpt.csz = curs.city clipped, ", ", curs.state, "  ", curs.zip clipped
    let rpt.country = curs.country
    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "cust_code" then
        # this is the area in the code where dunning messages could
        # be prepared & put into rpt.coment
        let rpt.coment = ""  # blank out the "CONTINUED ON NEXT PAGE" message
        let rpt.plspay = cust_total using "----,---.&&"
        whenever error continue
        update strcustr set (stmt_date, stmt_amount) = (entry_date, cust_total)
            where cust_code = curs.cust_code
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            call fg_err(1)  # Cannot update customer record - aborting
            call ml_output()
            finish report report1
            exit program(1)
        end if
        return
    end if

    if group_key = "last_row" then
        # insert "on last row" code here
        return
    end if

end function
# after_group()

#######################################################################
function clear_line()
#######################################################################
# This function blanks out all rpt variables defined in the
# "on every row" section of the report
#
    let rpt.act_date = ""
    let rpt.code1 = ""
    let rpt.ref1 = ""
    let rpt.chgs = ""
    let rpt.crdt = ""
    let rpt.bal = ""
    let rpt.code2 = ""
    let rpt.ref2 = ""
    let rpt.amt = ""
end function
# clear_line()

#######################################################################
function balance_forward()
#######################################################################
# This function processes rows for balance forward customers
#
    define
        quote  char(1),        #storage for the " character
        bf record
            doc_type like strtranr.doc_type,
            doc_date like stxtranr.doc_date,
            doc_no like stxtranr.doc_no,
            orig_doc like strtranr.inv_chk_no,
            amount like stractvd.amount
        end record,
        cr_doc_no like stxtranr.doc_no, # hold doc no for pmnt accumulation
        cr_amount like stractvd.amount  # pmnt accumulator

    let quote = "\""

    if curs.stmt_date != "01/01/80"
    then
        let rpt.act_date = curs.stmt_date using "mm/dd/yy"
    end if
    let rpt.code1 = "BF"
    let rpt.ref1 = "BAL FWD"
    let cust_total = curs.acct_bal

    # grab all trx's since last stmt date into the bf cursor
    declare bf_curs scroll cursor for
        select
            strtranr.doc_type,
            stxtranr.doc_date,
            stxtranr.doc_no,
            strtranr.inv_chk_no,
            stractvd.amount
        from stxtranr, stractvd, strtranr
        where
            stxtranr.ref_code = curs.cust_code and
            stxtranr.doc_date > curs.stmt_date and
            stxtranr.doc_no = stractvd.doc_no and
            stxtranr.orig_journal = stractvd.orig_journal and
            stractvd.act_type != "D" and
            stxtranr.doc_no = strtranr.doc_no and
            stxtranr.orig_journal = strtranr.orig_journal
        order by stxtranr.doc_date, stxtranr.doc_no
    open bf_curs

    fetch first bf_curs into bf.*  # compute statement beginning balance

    while sqlca.sqlcode != notfound
        let cust_total = cust_total - bf.amount
        fetch next bf_curs into bf.*
    end while
    let rpt.bal = cust_total using "----,---.&&"

    fetch first bf_curs into bf.*  # create a line for each unit of activity
    while sqlca.sqlcode != notfound
       if bf.doc_date <= entry_date
       then
           # send prev. line to report...
           call ml_output()
           call clear_line()
           # set rpt line variables
           let rpt.act_date = bf.doc_date using "mm/dd/yy"
           let rpt.ref1 = bf.orig_doc[1,7]
           if bf.doc_type = "IN"
           then
               if bf.orig_doc[1,2] = "FC"
               then
                   let rpt.code1 = "FC"
               else
                   let rpt.code1 = "IN"
               end if
               let rpt.chgs = bf.amount using "----,---.&&"
               let cust_total = cust_total + bf.amount
           end if
           if bf.doc_type = "CM"
           then
               let rpt.code1 = "CM"
               let rpt.crdt = 0 - bf.amount using "----,---.&&"
               let cust_total = cust_total + bf.amount
           end if
           if bf.doc_type = "DM"
           then
               let rpt.code1 = "DM"
               let rpt.chgs = bf.amount using "----,---.&&"
               let cust_total = cust_total + bf.amount
           end if
           if bf.doc_type = "CR" # accum amts for cash rcpt into one line
           then
               let cr_amount = 0
               let cr_doc_no = bf.doc_no
               while sqlca.sqlcode != notfound
                   if bf.doc_type = "CR" and bf.doc_no = cr_doc_no
                   then
                       let cr_amount = cr_amount + bf.amount
                   else
                       exit while
                   end if
                   fetch next bf_curs into bf.*
               end while
               let rpt.code1 = "P"
               let rpt.crdt = 0 - cr_amount using "----,---.&&"
               let cust_total = cust_total + cr_amount
               let rpt.bal = cust_total using "----,---.&&"
               continue while
           end if
           let rpt.bal = cust_total using "----,---.&&"
           fetch next bf_curs into bf.*
       else
           fetch next bf_curs into bf.*
       end if
    end while
    let sqlca.sqlcode = 0
    close bf_curs

    # print the new balance...
    call ml_output()
    call clear_line()
    let rpt.act_date = entry_date
    let rpt.code1 = "NB"
    let rpt.code2 = "BF"
    let rpt.ref1 = "NEW BAL"
    let rpt.ref2 = "BAL FWD"
    let rpt.bal = cust_total using "----,---.&&"
    let rpt.amt = rpt.bal

end function
# balance_forward()

#######################################################################
function open_item()
#######################################################################
# This function processes rows for open item customers
#

    define
        first_line smallint,             # first line flag
        p_ropend record like stropend.*

    let first_line = true
    if curs.on_acct_amt != 0
    then
        let rpt.code1 = "OA"
        let rpt.code2 = "OA"
        let rpt.ref1 = "ON ACCT"
        let rpt.ref2 = "ON ACCT"
        let cust_total = cust_total + curs.on_acct_amt
        let rpt.chgs = curs.on_acct_amt using "----,---.&&"
        let rpt.bal = cust_total using "----,---.&&"
        let rpt.amt = curs.on_acct_amt using "----,---.&&"
        let first_line = false
    end if

    # grab all open items into oi...
    declare oi_curs cursor for
        select * from stropend
        where stropend.cust_code = curs.cust_code
            and stropend.balance != 0
            and stropend.inv_date <= entry_date
        order by stropend.inv_date, stropend.doc_no
    foreach oi_curs into p_ropend.*
        # send prev. line to report...
        if first_line = false
        then
            call ml_output()
            call clear_line()
        end if
        # set rpt line variables
        let rpt.act_date = p_ropend.inv_date using "mm/dd/yy"
        let rpt.code1 = "IN"
        if p_ropend.inv_no[1,2] = "FC"
        then
            let rpt.code1 = "FC"
        end if
        let rpt.code2 = rpt.code1
        let rpt.ref1 = p_ropend.inv_no[1,7]
        let rpt.ref2 = rpt.ref1
        let rpt.chgs = p_ropend.balance using "----,---.&&"
        let rpt.bal = p_ropend.balance using "----,---.&&"
        let rpt.amt = p_ropend.balance using "----,---.&&"
        let cust_total = cust_total + p_ropend.balance
        let first_line = false
    end foreach
    close oi_curs
end function
# open_item()
